package com.noodlesandwich.rekord;

import java.util.Set;
import org.pcollections.HashTreePMap;
import org.pcollections.HashTreePSet;
import org.pcollections.PMap;

public final class Properties {
    private final PMap<Key<?, ?>, Object> properties;
    private final PMap<Key<?, ?>, Key<?, ?>> actualKeys;

    public Properties() {
        this(HashTreePMap.<Key<?, ?>, Object>empty(), HashTreePMap.<Key<?, ?>, Key<?, ?>>empty());
    }

    private Properties(PMap<Key<?, ?>, Object> properties, PMap<Key<?, ?>, Key<?, ?>> actualKeys) {
        this.properties = properties;
        this.actualKeys = actualKeys;
    }

    public Object get(Key<?, ?> key) {
        return properties.get(key);
    }

    public boolean contains(Key<?, ?> key) {
        return properties.containsKey(key);
    }

    @SuppressWarnings("unchecked")
    public <T> Set<Key<? super T, ?>> keys() {
        return (Set) HashTreePSet.from(actualKeys.values());
    }

    public <V> Properties with(Property<?, V> property) {
        return new Properties(properties.plus(property.key(), property.value()),
                              actualKeys.plus(property.key(), property.actualKey()));
    }

    public Properties without(Key<?, ?> key) {
        return new Properties(properties.minus(key),
                              actualKeys.minus(key));
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Properties)) return false;

        Properties other = (Properties) o;
        return properties.equals(other.properties);

    }

    @Override
    public int hashCode() {
        return properties.hashCode();
    }

    @Override
    public String toString() {
        return properties.toString();
    }
}
